# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2003, 2004 Peter Backlund
# Copyright (C) 2004 Thorsten Leemhuis
# Copyright (C) 2006, 2007 Van Assche Alphonse
# Copyright (C) 2006-2009 Stewart Adam
"""Module to configure the X configuration file

REMEMBER:
- These are the main functions
- Call all these functions at once, then the Glx ones later on.
  We don't want the contents of xorg.conf changing on us without a re-read!
"""
import os
import commands
import string
import xf86config

from livnaConfigDisplay import livnaConfigDisplayError
from livnaConfigDisplay.const import *
from livnaConfigDisplay import Utils


class LivnaConfigDisplay:
    """Module to configure the X configuration file"""

    def __init__(self, isVerbose=False):
        """Initialize."""
        Utils.setVerbose(isVerbose)
        try:
            (self.xconfig, self.xconfigpath) = self.loadXconfig()
        except:
            Utils._verbose(XORG_NOT_FOUND)
            Utils.writeFile(XORG_CONF_FILE, Utils.getSampleXorgConf())
            (self.xconfig, self.xconfigpath) = self.loadXconfig()

    def doAllBackups(self):
        """Backup configuration files"""
        currfile = GDM_CUSTOMCONF_FILE
        Utils.backupFile(GDM_CUSTOMCONF_FILE, BACKUP_NAME)
        currfile = KDM_CUSTOMCONF_FILE
        Utils.backupFile(KDM_CUSTOMCONF_FILE, BACKUP_NAME)
        currfile = self.xconfigpath
        Utils.backupFile(self.xconfigpath, BACKUP_NAME)

    def doAllRestores(self):
        """Restore configuration files"""
        if XGL_EXISTS:
            if GDM_EXISTS:
                currfile = GDM_CUSTOMCONF_FILE
                Utils.restoreBackup(GDM_CUSTOMCONF_FILE, BACKUP_NAME)
            if KDM_EXISTS:
                currfile = KDM_CUSTOMCONF_FILE
                Utils.restoreBackup(KDM_CUSTOMCONF_FILE, BACKUP_NAME)
        currfile = self.xconfigpath
        Utils.restoreBackup(self.xconfigpath, BACKUP_NAME)

    def getModTopDir(self):
        """Return the module directory depending on x86_64 or not"""
        if os.uname()[4] == "x86_64" :
            return "/usr/lib64/xorg/modules"
        else:
            return "/usr/lib/xorg/modules"

    def loadXconfig(self):
        """Load the X configuration file"""
        (self.xconfig, self.xconfigpath) = xf86config.readConfigFile()
        # Create a default files section in case we have none ...
        if self.xconfig.files == None:
            self.xconfig.files = xf86config.XF86ConfFiles()
        # ... with default module path
        if self.xconfig.files.module == None:
            self.xconfig.files.module = self.getModTopDir()
        return (self.xconfig, self.xconfigpath)
 
    def addModule(self, moduleName):
        """Adds a module to the Modules section. The module will only be listed once."""
        if self.xconfig.modules == None:
            self.xconfig.modules = xf86config.XF86ConfModule()
        else:
            self.removeModule(moduleName)
        m = xf86config.XF86ConfLoad()
        m.name = moduleName
        self.xconfig.modules.load.insert(m)
    
    def removeModule(self, moduleName):
        """Removes a module from the Modules section."""
        i = 0
        if not self.xconfig.modules == None:
            for m in self.xconfig.modules.load:
                if (m.name.lower() == moduleName.lower()):
                    self.xconfig.modules.load.remove(i)
                i += 1

    def toggleDriver(self, oldDriver, newDriver):
        """Set driver to newDriver where
        driver is oldDriver
        """
        if (oldDriver == "radeon") :
            for dev in self.xconfig.device:
                if (dev.driver == "radeon") or (dev.driver == "ati"):
                    dev.driver = newDriver
        else:
            for dev in self.xconfig.device:
                if (dev.driver == oldDriver):
                    dev.driver = newDriver

    def addOption(self, driver, addOptName, addOptVal):
        """Add option to driver device section where
        if dev.driver is driver
        """
        for dev in self.xconfig.device:
            if (dev.driver == driver):
                res = filter (lambda o: o.name == addOptName, dev.options)
                if len(res) == 0:
                    dev.options.insert(xf86config.XF86Option (addOptName,addOptVal))

    def removeOption(self, driver, addOptName):
        """Remove option to driver device section where
        if dev.driver is driver.
        """
        for dev in self.xconfig.device:
            if (dev.driver == driver):
                optcount = -1
                for opt in dev.options:
                    optcount += 1
                    if opt.name == addOptName:
                        dev.options.remove(optcount)
    
    def setDefaultDepth(self, newdepth):
        """Set Default Color Depth"""
        for scr in self.xconfig.device:
            olddepth = scr.defaultdepth
            if scr.defaultdepth == newdepth:
                continue
            scr.defaultdepth = newdepth
            for dsp in scr.display:
                if dsp.depth == olddepth:
                    dsp.depth = newdepth
                elif dsp.depth == newdepth:
                    dsp.depth = olddepth
                    
    def onlyHasModule(self, module):
        """Is it the only module?"""
        if not self.xconfig.modules == None:
            s = sum([m.name.lower() == module for m in self.xconfig.modules.load])
            return s == len(self.xconfig.modules.load)  
    
    def addModulePath(self, newPathEntry, topModDir):
        """This will add an entry to ModulePath section, with previous entries untouched."""
        prevModPath = []
        # check if section exist
        if (self.xconfig.files != None):
            # Check for existing ModulePath
            if (self.xconfig.files.module != None):
                prevModPath = string.split(self.xconfig.files.module, ",")
        else:
            # add a default XF86ConfFiles instance
            self.xconfig.files = xf86config.XF86ConfFiles()
        # First, add the default module dirs. We add the dirs in
        # reversed order, and reverse the list at the end. 
        newModPath = [topModDir] 
        #newModPath.append(TOP_MOD_DIR + "/extensions")
        for i in range(len(prevModPath)):
            mp = prevModPath[i]
            # Remove trailing "/", in case the config file
            # has been hand-edited
            if mp[len(mp) - 1] == "/":
                mp = mp[:(len(mp) - 1)]
            # Add to new module path
            if not mp in newModPath and mp != (topModDir + "/extensions"):
                newModPath.append(mp)
        # Add new path entry
        if not (topModDir + newPathEntry) in newModPath:
            newModPath.append(topModDir + newPathEntry)
        # Reverse list
        newModPath.reverse()
        self.xconfig.files.module = string.join(newModPath, ",")     

    def removeModulePath(self, modulePath, topModDir):
        """Remove a module path"""
        prevModPath = []
        # Check for existing ModulePath
        if (self.xconfig.files != None):
            prevModPath = string.split(self.xconfig.files.module, ",")
        if (len(prevModPath) < 1):
            # ModulePath empty, do nothing.
            return
        newModPath = []
        for i in range(len(prevModPath)):
            mp = prevModPath[i]
            # Remove trailing "/", in case the config file
            # has been hand-edited
            if mp[len(mp) - 1] == "/":
                mp = mp[:(len(mp) - 1)]
            if mp != (topModDir + modulePath) and mp != (topModDir + "/extensions"):
                newModPath.append(mp)
        self.xconfig.files.module = string.join(newModPath, ",")

    def getOption(self, device, optionname):
        """Return the value for a given option"""
        res = filter (lambda o: o.name == optionname, device.options)
        if len (res) > 0:
            return res[0]
        else:
            option = xf86config.XF86Option(optionname)
            device.options.insert(option)
            return option

    def getDriver(self):
        """Returns the current Driver tag's value"""
        for dev in self.xconfig.device:
            driver = dev.driver
        return driver
