# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2006, 2007 Van Assche Alphonse
# Copyright (C) 2006-2009 Stewart Adam
"""
Module for configuring Xgl and AIGLX.
"""

"""
REMEMBER:
- No backups. This should be done in the code this calling _these_ functions.
"""

import xf86config
import os, os.path
import re, string

from livnaConfigDisplay import livnaConfigDisplayError
from const import *
import Utils
from ConfigDisplay import LivnaConfigDisplay
from ConfigFile import ConfigFile

class GlxConfig(LivnaConfigDisplay):
    """Functions for configuring Xgl and AIGLX"""

    def __init__(self, isVerbose=False):
        """Initialize."""
        LivnaConfigDisplay.__init__(self, isVerbose)
        Utils.setVerbose(isVerbose)
        self.vendor = Utils.getVendor()
        if not os.path.exists(STATUS_FILE):
            self.config = ConfigFile(STATUS_FILE)
            self.setupBasicConfig()
        else:
            self.config = ConfigFile(STATUS_FILE)
            # inititate sections.
            newConfig = False
            for i in ['Configuration', 'ManualOptions']:
                if not self.config.has_section(i):
                    self.config.add_section(i)
                    newConfig = True
            if newConfig == True:
                self.setupBasicConfig()

    def setupBasicConfig(self):
        """ Construct a basic config file based on current settings """
        self.storeconf()
        self.config.setOldDriver(self.getDriver())
        self.config.write()

    # GLX stuff
    def enableAiglx(self):
        """Enables AIGLX"""
        self.setComposite(True)
        self.setAiglxFlag(True)
        self.setKdmXgl(False)
        self.setGdmXgl(False)

    def enableXgl(self):
        """Enables Xgl"""
        self.setComposite(False)
        self.setAiglxFlag(False)
        self.setKdmXgl(True)
        self.setGdmXgl(True)

    def disableAllGlxStuff(self):
        """Disables AIGLX, Xgl and Composite"""
        self.setAiglxFlag(False)
        self.setComposite(False)
        self.setKdmXgl(False)
        self.setGdmXgl(False)

    def getActive(self):
        """Get active for bootup"""
        if self.config.getActive() == True:
            return True
        else:
            return False

    def setActive(self, bool):
        """Set active for bootup"""
        return self.config.setActive(bool)

    def storeconf(self):
        """Current configuration > Configuration file"""
        for i in ['Configuration', 'ManualOptions']:
            if not self.config.has_section(i):
                self.config.add_section(i)
        
        if not self.config.has_option('Configuration', 'Active'):
            self.config.setActive(True)
        ctype = self.getCurrentGLXConfig()
        self.config.setConfigType(ctype)
        if self.getGdmXgl():
            self.config.setGdmXgl(True)
        else:
            self.config.setGdmXgl(False)
        if self.getKdmXgl():
            self.config.setKdmXgl(True)
        else:
            self.config.setKdmXgl(False)
        if self.getAiglxFlag():
            self.config.setAiglxFlag(True)
        else:
            self.config.setAiglxFlag(False)
        if self.getComposite():
            self.config.setComposite(True)
        else:
            self.config.setComposite(False)
        self.config.write()

    def restoreconf(self):
        """Configuration file > Current configuration"""
        ctype = str(self.config.getConfigType()).lower()
        if ctype == 'aiglx':
            self.enableAiglx()
        elif ctype == 'xgl':
            self.enableXgl()
        elif ctype == 'noglx':
            self.disableAllGlxStuff()
        elif ctype == 'manual':
            if self.config.getGdmXgl() == True:
                self.setGdmXgl(True)
            else:
                self.setGdmXgl(False)
            if self.config.getKdmXgl() == True:
                self.setKdmXgl(True)
            else:
                self.setKdmXgl(False)
            if self.config.getAiglxFlag() == True:
                self.setAiglxFlag(True)
            else:
                self.setAiglxFlag(False)
            if self.config.getComposite() == True:
                self.setComposite(True)
            else:
                self.setComposite(False)

    def getCurrentGLXConfig(self):
        """This function return the X configuration as a string.
        Returned string: "aiglx" | "xgl" | "noglx" | "manual"
        """
        xorgConfStr = Utils.readFile(self.xconfigpath)
        if self.getGdmXgl() and self.getKdmXgl() and not self.getAiglxFlag() and not self.getComposite():
            return "xgl"
        elif self.getAiglxFlag() and self.getComposite() and not self.getKdmXgl() and not self.getGdmXgl():
            return "aiglx"
        elif not self.getGdmXgl() and not self.getKdmXgl() and not self.getAiglxFlag() and not self.getComposite():
            return "noglx"
        else:
            return "manual"

    ## KDM|GDM + XGL CONFIG ##

    def getKdmXgl(self):
        """Get KDM is configured for Xgl
        """
        if not os.path.exists(KDM_CUSTOMCONF_FILE):
            Utils._verbose(MSG_KDM_NOT_FOUND)
            return False
        kdmrcStr = Utils.readFile(KDM_CUSTOMCONF_FILE)
        if not re.search("\nServerCmd=/usr/bin/Xgl.*", kdmrcStr, re.IGNORECASE):
            return False
        # If nothing fails, it's true.
        return True

    def setKdmXgl(self, boolValue=True):
        """Set KDM is configured for Xgl"""
        # Do nothing if Xgl is not found on the system.
        if not XGL_EXISTS:
            Utils._verbose(MSG_XGL_NOT_FOUND)
            return False
        # Do nothing if KDM is not found on the system.
        if not KDM_EXISTS:
            Utils._verbose(MSG_KDM_NOT_FOUND)
            return False
        if self.vendor.lower().startswith('nvidia'):
            command = XGL_COMMAND_NVIDIA % ('ServerCmd=')
        elif self.vendor.lower().startswith('fglrx'):
            command = XGL_COMMAND_FGLRX % ('ServerCmd=')
        else:
            command = XGL_COMMAND_OTHER % ('ServerCmd=')
        kdmrcStr = Utils.readFile(KDM_CUSTOMCONF_FILE)
        # replace all command=/usr/bin/Xgl lines.
        if re.search("\nServerCmd=.*", kdmrcStr) or re.search("\n\#ServerCmd=.*", kdmrcStr):
            p = re.compile("\n.*ServerCmd=.*", re.IGNORECASE)
            if boolValue:
                kdmrcStr = p.sub("\n" + command, kdmrcStr)
            else:
                kdmrcStr = p.sub("\n#" + command, kdmrcStr)
        #else:
        #    return False
        Utils.writeFile(KDM_CUSTOMCONF_FILE, kdmrcStr)

    def getGdmXgl(self, GdmConfigFile = GDM_CUSTOMCONF_FILE):
        """Get GDM is configured for Xgl"""
        # Do nothing if Xgl is not found on the system.
        if not XGL_EXISTS:
            Utils._verbose(MSG_XGL_NOT_FOUND)
            return False
        if not GDM_EXISTS:
            Utils._verbose(MSG_GDM_NOT_FOUND)
            return False
        gdmCustomStr = Utils.readFile(GdmConfigFile)
        if not re.search("\ncommand=/usr/bin/Xgl.*", gdmCustomStr, re.IGNORECASE):
            return False
        gdmCustomStr = Utils.readFile(GdmConfigFile)
        if not re.search("\n" + GDM_XGL_CONFIG_LINE, gdmCustomStr):
            return False
        # If nothing fails, it's true.
        return True


    def setGdmXgl(self, boolValue=True, GdmConfigFile=GDM_CUSTOMCONF_FILE):
        """Set GDM is configured for Xgl. Sets the server definition and default server.
        """
        # Do nothing if Xgl is not found on the system.
        if not XGL_EXISTS:
            Utils._verbose(MSG_XGL_NOT_FOUND)
            return False
        # Do nothing if Xgl is not found on the system.
        if not GDM_EXISTS:
            Utils._verbose(MSG_GDM_NOT_FOUND)
            return False
        gdmCustomStr = Utils.readFile(GdmConfigFile)
        if boolValue:
            # Set right XGL command line...
            if self.vendor.lower().startswith('nvidia'):
                command = XGL_COMMAND_NVIDIA % ('command=')
            elif self.vendor.lower().startswith('fglrx'):
                command = XGL_COMMAND_FGLRX % ('command=')
            else:
                command = XGL_COMMAND_OTHER % ('command=')
            if re.search("\ncommand=/usr/bin/Xgl.*", gdmCustomStr):
                # Replace all command=/usr/bin/Xgl lines if it's found.
                p = re.compile("\ncommand=/usr/bin/Xgl.*", re.IGNORECASE)
                gdmCustomStr = p.sub("\n" + command, gdmCustomStr)
            else:
                # If not: Add it!
                toadd = "# Automatically configured by livna-config-display\n"
                toadd += "[server-Xgl]\n"
                toadd += "name=Xgl\n"
                toadd += command + "\n"
                toadd += "flexible=true\n"
                toadd += "chooser=false\n"
                toadd += "handled=true\n"
                toadd += "priority=0\n"
                gdmCustomStr += toadd
        else:
            # if a xgl server def is found, get rid of it
            if re.search("\n# Automatically configured by livna-config-display\n\[server-Xgl\]\n", gdmCustomStr):
                p = re.compile('\n# Automatically configured by livna-config-display\n\[server-Xgl\]\nname=Xgl\ncommand=/usr/bin/Xgl.*\n\
flexible=true(\n)?chooser=false\nhandled=true\npriority=0\n', re.IGNORECASE)
                replaceStr = "\n"
                gdmCustomStr = p.sub(replaceStr, gdmCustomStr)
        # Next: Enable/Disable Xgl as default GDM server.
        if (boolValue):
            if not re.search("\n" + GDM_XGL_CONFIG_LINE, gdmCustomStr):
                p = re.compile('\[servers\]\n', re.IGNORECASE)
                replaceStr = "[servers]\n" + GDM_XGL_CONFIG_LINE + "\n"
                gdmCustomStr = p.sub(replaceStr, gdmCustomStr)
        else:
            p = re.compile("\n" + GDM_XGL_CONFIG_LINE + ".*\n", re.IGNORECASE)
            gdmCustomStr = p.sub('\n', gdmCustomStr)
        if (boolValue): # longer timeout
            if not re.search("\nGdmXserverTimeout=", gdmCustomStr):
                p = re.compile('\[daemon\]\n', re.IGNORECASE)
                replaceStr = "[daemon]\nGdmXserverTimeout=60\n"
                gdmCustomStr = p.sub(replaceStr, gdmCustomStr)
        # Save changes
        Utils.writeFile(GdmConfigFile, gdmCustomStr)

    ## AIGLX FLAG ##

    def getAiglxFlag(self):
        """Get AIGLX flag"""
        try:
            option = self.getOption(self.xconfig.flags, "AIGLX").val.lower()
            if option == "off" or option == "false" or option == "disable" or \
               option == "0" or option == "No":
                return False
            else:
                return True
        except AttributeError: # fedora defaults to true when unspecified
            return True

    def setAiglxFlag(self, boolValue = False):
        """Set AIGLX flag to the given bool"""
        if not self.xconfig.flags:
            self.xconfig.flags = xf86config.XF86ConfFlags()
        if (boolValue):
            # We don't use this for now, it seems Composite isn't enabled by
            # default
            """id = 0
            for i in self.xconfig.flags.options:
                if i.name.upper() == "AIGLX":
                    self.xconfig.flags.options.remove(id)
                    Utils.writeXorgConf(self.xconfig, self.xconfigpath)
                    break
                else:
                    id = int(id + 1)"""
            # Instead, let's just force enable it
            val = "on"
            self.getOption(self.xconfig.flags, "AIGLX").val = val
        else:
            val = "off"
            self.getOption(self.xconfig.flags, "AIGLX").val = val
        # Final write at end
        Utils.writeXorgConf(self.xconfig, self.xconfigpath)

    ## COMPOSITE ##

    def getComposite(self):
        """Get status of Composite extension"""
        strFile = Utils.readFile(self.xconfigpath)
        p = re.compile('Section.*Extensions.*\n', re.IGNORECASE)
        if re.search(p, strFile):
            # If Extension section exists... We search for Disable commands
            p = re.compile('Option.*Composite.*Disable.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                return False
            p = re.compile('Option.*Composite.*Off.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                return False
            p = re.compile('Option.*Composite.*False.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                return False
            p = re.compile('Option.*Composite.*0.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                return False
            p = re.compile('Option.*Composite.*No.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                return False
        else:
            # It doesn't exist!
            # For now we return True since AIGLX = on by default
            pass
        return True

    def setComposite(self, boolValue=True):
        """Enable or disable Composite extension"""
        # Search for the Extensions line
        strFile = Utils.readFile(self.xconfigpath)
        p = re.compile('Section.*Extensions.*\n', re.IGNORECASE)
        if not re.search(p, strFile):
            # If not found, then what? We'll make one.
            extensions = '\nSection "Extensions"\n'
            extensions += '\tOption "Composite" "Enable"\n'
            extensions += 'EndSection\n'
            strFile += extensions
            # Write and re-read the xorg.conf
            Utils.writeFile(self.xconfigpath, strFile)
            (self.xconfig, self.xconfigpath) = xf86config.readConfigFile()
            strFile = Utils.readFile(self.xconfigpath)
        if boolValue:
            # Enable the Composite extension
            p = re.compile('.*Option.*Composite.*\n', re.IGNORECASE)
            strToReplace = '\tOption "Composite" "Enable"\n'
            strFile = p.sub(strToReplace, strFile)
        else:
            # Disable the Composite extension
            p = re.compile('.*Option.*Composite.*\n', re.IGNORECASE)
            strToReplace = '\tOption "Composite" "Disable"\n'
            strFile = p.sub(strToReplace, strFile)
        # Final write at end
        Utils.writeFile(self.xconfigpath, strFile)
        (self.xconfig, self.xconfigpath) = xf86config.readConfigFile()



