# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2003, 2004 Peter Backlund
# Copyright (C) 2004 Thorsten Leemhuis
# Copyright (C) 2006, 2007 Van Assche Alphonse
# Copyright (C) 2007-2009 Stewart Adam
"""
Utilities for livnaConfigDislay
"""

from livnaConfigDisplay import livnaConfigDisplayError
from const import *
from const import _

import shutil
import os, os.path
import commands
import types
#import inspect

verbose = False

def _verbose(str):
    """Write verbose text"""
    if verbose:
        print str

def setVerbose(isVerbose):
    """Enable/Disable verbose stuff"""
    if isVerbose:
        verbose = True
    else:
        verbose = False
            
def getKernelRelease():
    """ Get the current (running) kernel release """
    return os.uname()[2]

def getVendor():
    """ Return the installed driver, or return False """
    # Files first.
    majorVendor = getMajorVendor()
    for i in MODULE_LIST:
        if os.path.exists("/lib/modules/%s/extra/%s/%s.ko" % (getKernelRelease(), i, majorVendor)):
            return i
        elif os.path.exists("/lib/modules/%s/extra/%s.ko" % (getKernelRelease(), i)):
            return i
        elif os.path.exists("/lib/modules/%s/kernel/drivers/video/%s.ko" % (getKernelRelease(), i)):
            return i
    # All else fails: Check for RPMs.
    for i in MODULE_LIST:
        status, output = commands.getstatusoutput('rpm -q xorg-x11-drv-%s' % i)
        if status == os.EX_OK:
            return i
    return 'False'

def getMajorVendor():
    """ Return the installed vendor, or return False """
    # Files first.
    for i in MODULE_LIST:
        if i.lower().startswith('nvidia'):
            majorVendor = 'nvidia'
        elif i.lower().startswith('fglrx'):
            majorVendor = 'fglrx'
        # new if to detect files
        if os.path.exists("/lib/modules/%s/extra/%s/%s.ko" % (getKernelRelease(), i, majorVendor)):
            return majorVendor
        # for nvidia's .run packages:
        elif os.path.exists("/lib/modules/%s/extra/%s.ko" % (getKernelRelease(), i)):
            return majorVendor
        elif os.path.exists("/lib/modules/%s/kernel/drivers/video/%s.ko" % (getKernelRelease(), i)):
            return majorVendor
    # All else fails: Check for RPMs.
    for i in MODULE_LIST:
        status, output = commands.getstatusoutput('rpm -q xorg-x11-drv-%s' % i)
        if status == os.EX_OK:
            if i.lower().startswith('nvidia'):
                majorVendor = 'nvidia'
            elif i.lower().startswith('fglrx') or i.lower().startswith('catalyst'):
                majorVendor = 'fglrx'
            return majorVendor
    return 'False'

def runLdconfig():
    """ Runs ldconfig """
    try:
        os.system("/sbin/ldconfig")
    except:
        None

def getSampleXorgConf():
  """Returns a sample Xorg configuration for the given hardware"""
  majorVendor = getMajorVendor()
  if majorVendor == 'nvidia':
    stockDriver = 'nouveau'
  elif majorVendor == 'fglrx':
    stockDriver = 'radeon'
  else:
    stockDriver = 'vesa'
  return """
Section "Files"
        ModulePath   "%sxorg/modules"
EndSection

Section "ServerFlags"
        Option      "AIGLX" "on"
EndSection

Section "Device"
        Identifier  "Videocard0"
        Driver      "%s"
EndSection

Section "Extensions"
        Option      "Composite" "Enable"
EndSection
""" % (LIBDIR, stockDriver)

def checkIfKernelModuleIsLoaded(modulename):
    """Checks if a kernel module is loaded and tries to unload if unload = True"""
    cmd = "/sbin/lsmod | grep " + "\"^" + modulename + "\""
    moduleline = commands.getoutput(cmd)
    if len(moduleline) > 0:
        # module is loaded
        lineItems = moduleline.split()
        if len(lineItems) == 3:
            module, size, usage = moduleline.split()
        elif len(lineItems) == 4:
            module, size, usage, usedBy = moduleline.split()
            
        if usage == "0":
            # usage 0
            return 1
        else:
            return 2
        return 0
    
def checkForModuleAndRemove(modulename):
    """We cannot disable or enable the driver blindly -- the kernel-module 
    might be loaded, try to unload the module.otherwise let the init-script
    enable or disable the driver during next restart"""        
    ret = checkIfKernelModuleIsLoaded(modulename)
    if ret == 1:
        if os.system("/sbin/modprobe -r "+modulename) > 0 :
            return False
        elif ret == 2:
            return False
        return True
    
def readFile(filename):
    """Read a given file"""
    try:
        _verbose(_("Read file %s") % filename)
        fileHandle = open (filename, 'r')
        text = fileHandle.read()
        fileHandle.close()
    except OSError, (errno, strerror):
        raise livnaConfigDisplayError(MSG_ERROR_CANNOT_READ % (filename) + '\n' + MSG_TRACEBACK % (strerror))
    return text

def writeFile(filename, textToWrite):
    """Write a given string to file"""
    try:
        _verbose(_("Write file %s") % filename)
        fileHandle = open (filename, 'w')
        fileHandle.write(textToWrite)
        fileHandle.close()
    except IOError, (errno, strerror):
        raise livnaConfigDisplayError(MSG_ERROR_CANNOT_WRITE % (filename) + '\n' + MSG_TRACEBACK % (strerror))

def removeFile(sfile):
    """Remove a given file"""
    try:
        os.remove(sfile)
    except:
        print _("Error removing file '%s'... Ignoring.") % sfile
        pass

def writeXorgConf(xconfig, xconfigpath):
    """Write xorg.conf file"""
    try:
        xconfig.comment = "# Xorg configuration created by livna-config-display\n"
        _verbose(_("Write file %s") % xconfigpath)
        xconfig.write(xconfigpath)
    except:
        raise livnaConfigDisplayError(MSG_ERROR_CANNOT_WRITE % (xconfigpath))

def backupFile(filename, backupName):
    """Backup a given file"""
    try:
        _verbose(_("Backup '%s' ") % filename + " => " + filename + backupName)
        if os.path.exists(filename):
            shutil.copy(filename, filename + backupName)
        else:
            _verbose(_('Skipping backup of \'%s\', file was not found' % filename))
    except IOError, (errno, strerror):
        raise livnaConfigDisplayError(MSG_CONF_BACKUP_ERROR % (filename) + '\n' + MSG_TRACEBACK % (strerror))
    except:
        raise livnaConfigDisplayError(MSG_CONF_BACKUP_ERROR % (filename))

def restoreBackup(filename, backupName):
    """Backup a given file"""
    try:
        backupFullName = filename + backupName
        _verbose(_("Restore backup '%s' ") % backupFullName + " => " + filename)
        shutil.copy(backupFullName, filename)
    except IOError, (errno, strerror):
        raise livnaConfigDisplayError(MSG_CONF_RESTORE_ERROR % (backupFullName) + '\n' + MSG_TRACEBACK % (strerror))
    except:
        raise livnaConfigDisplayError(MSG_CONF_RESTORE_ERROR % (backupFullName))

"""def moduleClasses(mod):
    \"""Introspect a gived module for classes\"""
    def P(obj, m=mod.__name__, CT=types.ClassType):
        return (type(obj)==CT and obj.__module__==m)
    try:
        return inspect.getmembers(mod, P)[0][1]
    except:
        return None

def getClass(classNameString):
    \"""Get the class for a given class name\"""
    return moduleClasses(__import__(classNameString))"""

