# -*- coding: UTF-8 -*-
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright (C) 2006, 2007 Van Assche Alphonse
# Copyright (C) 2006, 2007 Stewart Adam
"""
Configuration dialog (GUI) for livnaConfigDisplay
"""


"""
REMEMBER:
- Only GUI stuff. All functions that need to be called (apart form the initial callbacks) should be in other files.
- No backups. This should be done in the code this _calling these_ functions.
"""

import gtk, gtk.glade, gobject
import os
from livnaConfigDisplay.const import *
from livnaConfigDisplay.GlxConfig import *
from livnaConfigDisplay import livnaConfigDisplayError
from livnaConfigDisplay import Utils
from livnaConfigDisplay.const import _

class UI(gtk.glade.XML):
    """ Base class for UIs loaded from glade. """
    
    def __init__(self, filename, rootname, domain=False):
        """ Initialize a new instance.
        `filename' is the name of the .glade file containing the UI hierarchy.
        `rootname' is the name of the topmost widget to be loaded.
        `gladeDir' is the name of the directory, relative to the Python
        path, in which to search for `filename'. """
        if domain:
            gtk.glade.XML.__init__(self, filename, None, domain)
        else:
            gtk.glade.XML.__init__(self, filename, root=None)
        self.filename = filename
        self.root = self.get_widget(rootname)

    def __getattr__(self, name):
        """ Look up an as-yet undefined attribute, assuming it's a widget. """
        result = self.get_widget(name)
        if result is None:
            raise AttributeError("Can't find widget %s in %s.\n" %
                                 (`name`, `self.filename`))
        
        # Cache the widget to speed up future lookups.  If multiple
        # widgets in a hierarchy have the same name, the lookup
        # behavior is non-deterministic just as for libglade.
        setattr(self, name, result)
        return result

class Controller:
    """ Base class for all controllers of glade-derived UIs. """
    def __init__(self, ui):
        """ Initialize a new instance.
        `ui' is the user interface to be controlled. """
        self.ui = ui
        self.ui.signal_autoconnect(self._getAllMethods())

    def _getAllMethods(self):
        """ Get a dictionary of all methods in self's class hierarchy. """
        result = {}

        # Find all callable instance/class attributes.  This will miss
        # attributes which are "interpreted" via __getattr__.  By
        # convention such attributes should be listed in
        # self.__methods__.
        allAttrNames = self.__dict__.keys() + self._getAllClassAttributes()
        for name in allAttrNames:
            value = getattr(self, name)
            if callable(value):
                result[name] = value
        return result

    def _getAllClassAttributes(self):
        """ Get a list of all attribute names in self's class hierarchy. """
        nameSet = {}
        for currClass in self._getAllClasses():
            nameSet.update(currClass.__dict__)
        result = nameSet.keys()
        return result

    def _getAllClasses(self):
        """ Get all classes in self's heritage. """
        result = [self.__class__]
        i = 0
        while i < len(result):
            currClass = result[i]
            result.extend(list(currClass.__bases__))
            i = i + 1
        return result

class Callbacks(Controller):
    """ All Glade Callbacks """
    def __init__(self, CD):
        # Create and ui object contains the widgets.
        ui = UI('/usr/share/livna-config-display/livna-config-display.glade' , 'main', I18N_DOMAIN)
        # init the Controller Class to connect signals.
        Controller.__init__(self, ui)
        self.CD = CD

    def quit(self, *args):
        """ Quit """
        gtk.main_quit()
        return False

    def hide(self, widget):
        """ Dont destroy me! """
        widget.hide()

    def on_CancelButton_clicked(self, widget):
        """ Cancel button, do nothing and close down """
        self.quit()

    def on_ApplyButton_clicked(self, widget):
        """ Apply: Take action!"""
        try:
            # honor active=true|false
            if self.ui.DriverEnableCheck.get_active():
                self.CD.Glx.setActive(True)
            else:
                self.CD.Glx.setActive(False)
            self.CD.Glx.config.write()
            if self.ui.DriverEnableCheck.get_active():
                # if Manual
                if self.ui.ManualRadio.get_active():
                    # Composite?
                    if self.ui.CompositeCheck.get_active():
                        self.CD.Glx.setComposite(True)
                    else:
                        self.CD.Glx.setComposite(False)
                    # AIGLX server flag?
                    if self.ui.AIGLXFlagCheck.get_active():
                        self.CD.Glx.setAiglxFlag(True)
                    else:
                        self.CD.Glx.setAiglxFlag(False)
                    # Xgl in GDM?
                    if self.ui.XglGDMCheck.get_active():
                        self.CD.Glx.setGdmXgl(True)
                    else:
                        self.CD.Glx.setGdmXgl(False)
                    # Xgl in KDM?
                    if self.ui.XglKDMCheck.get_active():
                        self.CD.Glx.setKdmXgl(True)
                    else:
                        self.CD.Glx.setKdmXgl(False)
                # if AIGLX
                elif self.ui.AIGLXRadio.get_active():
                    self.CD.Glx.enableAiglx()
                # if Xgl
                elif self.ui.XglRadio.get_active():
                    self.CD.Glx.enableXgl()
                # if None
                elif self.ui.DisableRadio.get_active():
                    self.CD.Glx.disableAllGlxStuff()
                # Never happens, but handle anyways
                else:
                    # use Debug-1 so we can track where it failed to
                    raise livnaConfigDisplayError('Please report a bug against livna-config-display: Debug-1')
                self.CD.Glx.storeconf()
                dia = InfoDia(self.ui, '%s\n%s' % (MSG_CONF_APPLIED, MSG_RESTART_X))
                dia.run()
                dia.destroy()
        except livnaConfigDisplayError, (errMsg):
            errMsg = '\n' + str(errMsg)
            dia = ErrorDia(self.ui, MSG_CONF_APPLY_ERROR + '\n' + MSG_TRACEBACK % (str(errMsg)))
            dia.run()
            dia.destroy()
        self.quit()

    def on_DriverEnableCheck_toggled(self, widget):
        """ Don't use lcd toggled """
        self.ui.notebook1.set_sensitive(widget.get_active())

    def on_ManualRadio_toggled(self, widget):
        """ Manual toggled """
        self.ui.table10.set_sensitive(True)
    def on_AIGLXRadio_toggled(self, widget):
        """ AIGLX toggled """
        if self.ui.AIGLXRadio.get_active:
            self.ui.table10.set_sensitive(False)
            self.ui.CompositeCheck.set_active(True)
            self.ui.AIGLXFlagCheck.set_active(True)
            self.ui.XglGDMCheck.set_active(False)
            self.ui.XglKDMCheck.set_active(False)
    def on_XglRadio_toggled(self, widget):
        """ Xgl toggled """
        if self.ui.XglRadio.get_active:
            self.ui.table10.set_sensitive(False)
            self.ui.CompositeCheck.set_active(False)
            self.ui.AIGLXFlagCheck.set_active(False)
            self.ui.XglGDMCheck.set_active(True)
            self.ui.XglKDMCheck.set_active(True)
    def on_DisableRadio_toggled(self, widget):
        """ None toggled """
        if self.ui.DisableRadio.get_active:
            self.ui.table10.set_sensitive(False)
            self.ui.CompositeCheck.set_active(False)
            self.ui.AIGLXFlagCheck.set_active(False)
            self.ui.XglGDMCheck.set_active(False)
            self.ui.XglKDMCheck.set_active(False)

    def _setOptions(self):
        """ Load the UI and set widgets based on system values """
        self.config = self.CD.Glx.getCurrentGLXConfig()
        self.ui.table10.set_sensitive(False)
        self.ui.XglRadio.set_sensitive(False)
        self.ui.XglGDMCheck.set_sensitive(False)
        self.ui.XglKDMCheck.set_sensitive(False)
        # autoconf types (comes first so no checks get reset)
        if self.config == "manual":
            self.ui.ManualRadio.set_active(True)
            if XGL_EXISTS:
                self.ui.XglRadio.set_sensitive(True)
                self.ui.XglGDMCheck.set_sensitive(True)
                self.ui.XglKDMCheck.set_sensitive(True)
        elif self.config == "aiglx":
            self.ui.AIGLXRadio.set_active(True)
        elif self.config == "xgl":
            self.ui.XglRadio.set_active(True)
        elif self.config == "noglx":
            self.ui.DisableRadio.set_active(True)
        else:
            print 'DEBUG: self.config: %s' % self.config
        if XGL_EXISTS:
            self.ui.XglRadio.set_sensitive(True)
            self.ui.XglGDMCheck.set_sensitive(True)
            self.ui.XglKDMCheck.set_sensitive(True)
        # Active?
        if self.CD.Glx.getActive():
            self.ui.DriverEnableCheck.set_active(True)
        else:
            self.ui.DriverEnableCheck.set_active(False)
        # Composite?
        if self.CD.Glx.getComposite():
            self.ui.CompositeCheck.set_active(True)
        else:
            self.ui.CompositeCheck.set_active(False)
        # AIGLX server flag?
        if self.CD.Glx.getAiglxFlag():
            self.ui.AIGLXFlagCheck.set_active(True)
        else:
            self.ui.AIGLXFlagCheck.set_active(False)
        # Xgl in GDM?
        if self.CD.Glx.getGdmXgl():
            self.ui.XglGDMCheck.set_active(True)
        else:
            self.ui.XglGDMCheck.set_active(False)
        # Xgl in KDM?
        if self.CD.Glx.getKdmXgl():
            self.ui.XglKDMCheck.set_active(True)
        else:
            self.ui.XglKDMCheck.set_active(False)

class ConfigDialog(Callbacks):
    """ The Configuration dialog """

    def __init__(self, CD):
        """ Initialize ConfigDialog """
        self.icon = ICON_PATH
        # enable/disable verbose mode
        self.CD = CD
        Callbacks.__init__(self, self.CD)
        self.ui.main.set_title(_('Livna Display Configuration'))
        self.vendor = Utils.getVendor()
        self.supports = [DISABLE_ALL]
        if self.vendor in ["nvidia", "nvidia-96xx", "nvidia-173xx",
                           "nvidia-newest", "nvidia-beta"]:
            self.supports.append('AIGLX')
            self.supports.append('Xgl')
            self.recommended = 'AIGLX'
        elif self.vendor == "nvidia-legacy":
            self.recommended = MSG_NONE
        elif self.vendor == "fglrx":
            self.supports.append('AIGLX')
            self.supports.append('Xgl')
            self.recommended = 'AIGLX'
        # No driver installed?
        elif self.vendor == 'False':
            self.ui.DriverNameLabel.set_text(MSG_NONE_FOUND)
            self.supports = ['Unknown']
            self.recommended = 'AIGLX'
            # Let them edit still...
            #self.ui.table7.set_sensitive(False)
            self.ui.DriverEnableCheck.set_sensitive(False)
            #self.ui.ApplyButton.set_sensitive(False)
        self.ui.ConfigSupportedLabel.set_text(', '.join(self.supports))
        self.ui.RecommendedConfigLabel.set_text(self.recommended)
        if self.vendor == 'False':
            dia = ErrorDia(self.ui, MSG_DRIVER_NOT_FOUND)
            dia.run()
            dia.destroy()
        else:
            self.ui.DriverNameLabel.set_text(self.vendor)
        self._setOptions()

class ErrorDia:
    def __init__(self, ui, message):
        self.ui = ui
        self.dia = self.ui.error_dia
        self.label = self.ui.ErrorLabel
        self.message = message

    def destroy(self):
        return self.dia.hide()

    def run(self):
        self.dia.show_all()
        self.label.set_text(self.message)
        self.label.set_use_markup(True)
        return self.dia.run()

class InfoDia:
    def __init__(self, ui, message):
        self.ui = ui
        self.dia = self.ui.info_dia
        self.label = self.ui.InformationLabel
        self.message = message

    def destroy(self):
        return self.dia.hide()

    def run(self):
        self.dia.show_all()
        self.label.set_text(self.message)
        self.label.set_use_markup(True)
        return self.dia.run()


